#include <htc.h>
#ifndef __SW_SPI_H
#define __SW_SPI_H

/* PIC18 Software SPI library header
 *
 * To use the software spi routines, the user must define
 * the port and tris register for each of the CS, DIN, DOUT,
 * and SCK pins.  The SPI mode must also be defined (MODE0,
 * MODE1, MODE2, MODE3).
 *
 * Define the port and pin for each of the software SPI pins
 *  - Chip select pin CS must have a port and tris definition.
 *  - Data in pin DIN must have a port and tris definition.
 *  - Data out pin DOUT must have a port and tris definition.
 *  - Clock pin SCK must have a port and tris definition.
 */

#if defined (_18F13K50) || defined (_18F14K50) ||\
    defined (_18LF13K50) || defined (_18LF14K50) ||\
    defined (_18F13K20) || defined (_18F14K20) ||\
    defined (_18F13K22) || defined (_18F14K22) ||\
    defined (_18LF13K22) || defined (_18LF14K22)

#define SW_CS_PIN         PORTDbits.RD2      // Chip Select
#define TRIS_SW_CS_PIN    TRISD2
#define SW_DIN_PIN        PORTDbits.RD1     // Data in
#define TRIS_SW_DIN_PIN   TRISD1
#define SW_DOUT_PIN       PORTDbits.RD3    // Data out
#define TRIS_SW_DOUT_PIN  TRISD3
#define SW_SCK_PIN        PORTDbits.RD0     // Clock
#define TRIS_SW_SCK_PIN   TRISD0
 
#else
#define SW_CS_PIN         PORTDbits.RD2      // Chip Select
#define TRIS_SW_CS_PIN    TRISD2
#define SW_DIN_PIN        PORTDbits.RD1     // Data in
#define TRIS_SW_DIN_PIN   TRISD1
#define SW_DOUT_PIN       PORTDbits.RD3    // Data out
#define TRIS_SW_DOUT_PIN  TRISD3
#define SW_SCK_PIN        PORTDbits.RD0     // Clock
#define TRIS_SW_SCK_PIN   TRISD0
#endif
// Define the mode for software SPI
// Refer to the SPI module for PIC17C756 for definitions of CKP and CKE
// Only one mode can be uncommented, otherwise the software will not work
#define _MODE0  // CKP=0,CKE=0
//#define _MODE1  // CKP=1,CKE=0
//#define _MODE2  // CKP=0,CKE=1
//#define _MODE3  // CKP=1,CKE=1

/* OpenSWSPI  (SWOpenSPI)
 * Configure Software SPI I/O pins
 */
void OpenSWSPI(void);
#define SWOpenSPI OpenSWSPI

/* WriteSWSPI  (SWWriteSPI)
 * Write a data byte to the SPI, return byte read
 */
char WriteSWSPI(char);
#define SWWriteSPI WriteSWSPI

/* SetCSSWSPI  (SWSetCSSPI)
 * Sets the CS pin
 */
#define SetCSSWSPI( )	SW_CS_PIN=1   // Sets the CS pin
#define SWSetCSSPI SetCSSWSPI

/* ClearCSSWSPI  (SWClearCSSPI)
 * Clears the CS pin
 */
#define ClearCSSWSPI( ) SW_CS_PIN=0  // Clears the CS pin
#define SWClearCSSPI ClearCSSWSPI

/* putcSWSPI  (SWputcSPI)
 * putc is really write
 */
#define putcSWSPI WriteSWSPI
#define SWputcSPI putcSWSPI


#endif /* __SW_SPI_H */
