#include <htc.h>
#include "sw_spi.h"

/********************************************************************
*       Function Name:  WriteSWSPI                                  *
*       Return Value:   char: received data                         *
*       Parameters:     data: data to transmit                      *
*       Description:    This routine sets the CS pin high.          *
********************************************************************/
char WriteSWSPI( char output)
{
        char bitcount;                  // Bit counter
        static char input;

        bitcount = 8;                   // Do 8-bits
        input = output;

#if defined(_MODE0)                      // Mode 0
// SCK idles low
// Data output after falling edge of SCK
// Data sampled before rising edge of SCK
        SW_DOUT_PIN = (input&0x80);		// Set Dout to MSB of data
        NOP();					// Adjust for jitter
        NOP();
        while (bitcount--)
        {
                CARRY = SW_DIN_PIN?1:0;		// Set the carry bit according
                SW_SCK_PIN = 1;         	// Set the SCK pin
		input=input<<1 | input >>7;	// Rotate the carry into the data byte
                NOP();                  	// Produces a 50% duty cycle clock
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                SW_SCK_PIN = 0;         	// Clear the SCK pin
        	SW_DOUT_PIN = (input&0x80);	// Set Dout to MSB of data
        } 
#endif

#if defined(_MODE1)                      // Mode1
// SCK idles high
// Data output after rising edge of SCK
// Data sampled before falling edge of SCK
        SW_DOUT_PIN = (input&0x80);		// Set Dout to MSB of data
        NOP();					// Adjust for jitter
        NOP();
        while (bitcount--)
        {
                CARRY = SW_DIN_PIN?1:0;		// Set the carry bit according
                SW_SCK_PIN = 0;         	// Set the SCK pin
		input=input<<1 | input >>7;	// Rotate the carry into the data byte
                NOP();                  	// Produces a 50% duty cycle clock
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                NOP();
                SW_SCK_PIN = 1;         	// Clear the SCK pin
        	SW_DOUT_PIN = (input&0x80);	// Set Dout to MSB of data
        } 
#endif

#if defined(_MODE2)                      // Mode2
// SCK idles low
// Data output after rising edge of SCK
// Data sampled after falling edge of SCK
        while (bitcount--)
        {
                SW_SCK_PIN = 1;         	// Set the SCK pin
        	SW_DOUT_PIN = (input&0x80);		// Set Dout to MSB of data
                CARRY = SW_DIN_PIN?1:0;		// Set the carry bit according
                SW_SCK_PIN = 0;         	// Clear the SCK pin
		input=input<<1 | input >>7;	// Rotate the carry into the data byte
                NOP();                  	// Produces a 50% duty cycle clock
                NOP();
                NOP();
                NOP();
                NOP();
        } 

#endif

#if defined(_MODE3)                      // Mode 3
// SCK idles high
// Data output after falling edge of SCK
// Data sampled before rising edge of SCK
        do                              // Loop 8 times
        {
                SW_SCK_PIN = 0;         // Clear the SCK pin
                SW_DOUT_PIN = 0;        // Set the Dout pin according to 
                if(input&0x80)          // the MSB of data
                        SW_DOUT_PIN = 1;
                CARRY = 0;       // Set the carry bit according to
                if(SW_DIN_PIN)          // the state of the Din pin
                        CARRY = 1;
                SW_SCK_PIN = 1;         // Set the SCK pin
                Rlcf(input);            // Rotate the carry into the data byte
                NOP();                  // Produces a 50% duty cycle clock
                NOP();
                NOP();
                NOP();
                NOP();
                bitcount--;             // Count loop iterations
        } while(bitcount);
        while (bitcount--)
        {
                SW_SCK_PIN = 0;         	// Set the SCK pin
        	SW_DOUT_PIN = (input&0x80);		// Set Dout to MSB of data
                CARRY = SW_DIN_PIN?1:0;		// Set the carry bit according
                SW_SCK_PIN = 1;         	// Clear the SCK pin
		input=input<<1 | input >>7;	// Rotate the carry into the data byte
                NOP();                  	// Produces a 50% duty cycle clock
                NOP();
                NOP();
                NOP();
                NOP();
        } 

#endif

        return(input);                  // Return the received data
}

